#ifndef CSAPEX_PLUGIN_H
#define CSAPEX_PLUGIN_H

/// COMPONENT
#include "background_remover.h"
#include <csapex/csapex_fwd.h>

/// PROJECT
#include <csapex/model/node.h>
#include <csapex_vision/cv_mat_message.h>

/// SYSTEM
#include <mutex>

namespace csapex
{

class BackgroundSubtraction : public Node
{
public:
    enum Method {
        OpenCV,
        Simple,
        Adaptive
    };

public:
    BackgroundSubtraction();

    void setup(csapex::NodeModifier& node_modifier) override;
    void setupParameters(Parameterizable& parameters);

    void resetRemover();
    void updateRemover();
    void updateSettings();
    void setBackground();

    virtual void process() override;

private:
    std::shared_ptr<background_subtraction::BackgroundRemover> remover;
    std::mutex remover_mutex;

    connection_types::CvMatMessage::Ptr bg_frame;

    Input* input_;
    Output* out_masked_;
    Output* out_mask_;
    Output* out_bg_;
};

}

#endif // CSAPEX_PLUGIN_H
