#ifndef CROSS_MESSAGE_H
#define CROSS_MESSAGE_H

/// PROJECT
#include <csapex/msg/message.h>
#include <csapex_core_plugins/point_message.h>

namespace csapex {
namespace connection_types {


struct CrossMessage : public Message
{
    typedef std::shared_ptr<CrossMessage> Ptr;
    typedef std::shared_ptr<CrossMessage const> ConstPtr;

    CrossMessage(const std::string& frame_id, Message::Stamp stamp);

    virtual ConnectionType::Ptr clone() const override;
    virtual ConnectionType::Ptr toType() const override;

    enum Type {
        //! Message contains the position of the intersection point of the cross.
        INTERSECTION,
        //! Message contains the center of a ROI which may contains a cross (...which is not yet visible e.g. because
        //! it is too far away).
        REGION_CENTER,
        //! Message contains a point on the vertical middle-line of the target
        LINE
    };

    //! Position of the cross in the image
    PointMessage position;
    //! Type of the message. See enum Type for possible values.
    Type type;
};


/// TRAITS
template <>
struct type<CrossMessage> {
    static std::string name() {
        return "Cross";
    }
};

template <>
inline std::shared_ptr<CrossMessage> makeEmpty<CrossMessage>()
{
    return std::shared_ptr<CrossMessage>(new CrossMessage("/camera", 0));
}

}
}


/// YAML
namespace YAML {
    template<>
    struct convert<csapex::connection_types::CrossMessage> {
      static Node encode(const csapex::connection_types::CrossMessage& rhs);
      static bool decode(const Node& node, csapex::connection_types::CrossMessage& rhs);
    };
}

#endif // CROSS_MESSAGE_H
