#ifndef TARGET_MESSAGE_H
#define TARGET_MESSAGE_H

/// PROJECT
#include <csapex/msg/message.h>
#include <csapex_transform/transform_message.h>

namespace csapex {
namespace connection_types {


struct TargetMessage : public Message
{
    typedef std::shared_ptr<TargetMessage> Ptr;
    typedef std::shared_ptr<TargetMessage const> ConstPtr;

    TargetMessage(const std::string &frame_id, Stamp stamp);

    virtual ConnectionType::Ptr clone() const override;
    virtual ConnectionType::Ptr toType() const override;

public:
    enum DetectorType {
        UNDEFINED,
        //! Target-cross was detected.
        CROSS,
        //! No cross but only a vertical line.
        VERTICAL_LINE,
        //! No lines detected, only ROI center.
        REGION_CENTER
    };

    TransformMessage transformation;

    DetectorType detector_type;
};



/// TRAITS
template <>
struct type<TargetMessage> {
    static std::string name() {
        return "Target";
    }
};

template <>
inline std::shared_ptr<TargetMessage> makeEmpty<TargetMessage>()
{
    return std::shared_ptr<TargetMessage>(new TargetMessage("/camera", 0));
}

}
}

/// YAML
namespace YAML {
template<>
struct convert<csapex::connection_types::TargetMessage> {
  static Node encode(const csapex::connection_types::TargetMessage& rhs);
  static bool decode(const Node& node, csapex::connection_types::TargetMessage& rhs);
};
}

#endif // TARGET_MESSAGE_H
