/// HEADER
#include <sick14/cross_message.h>

/// PROJECT
#include <csapex/utility/assert.h>
#include <csapex/utility/register_msg.h>

CSAPEX_REGISTER_MESSAGE(csapex::connection_types::CrossMessage)

using namespace csapex;
using namespace connection_types;

CrossMessage::CrossMessage(const std::string& frame_id, Message::Stamp stamp)
    : Message (connection_types::type<CrossMessage>::name(), frame_id, stamp), position(stamp)
{
    // default value
    type = INTERSECTION;
}

ConnectionType::Ptr CrossMessage::clone() const
{
    Ptr new_msg(new CrossMessage(frame_id, stamp_micro_seconds));
    new_msg->position = position;
    new_msg->type = type;
    return new_msg;
}

ConnectionType::Ptr CrossMessage::toType() const
{
    Ptr new_msg(new CrossMessage(frame_id, 0));
    return new_msg;
}


/// YAML
namespace YAML {
Node convert<csapex::connection_types::CrossMessage>::encode(const csapex::connection_types::CrossMessage& rhs)
{
    Node node;
    node["position"] = rhs.position;
    node["type"] = (int) rhs.type;
    return node;
}

bool convert<csapex::connection_types::CrossMessage>::decode(const Node& node, csapex::connection_types::CrossMessage& rhs)
{
    if(!node.IsMap()) {
        return false;
    }
    rhs.position = node["position"].as<PointMessage>();
    rhs.type = (CrossMessage::Type) node["type"].as<int>();
    return true;
}
}
