#ifndef MASK_IMAGE_FROM_SCAN_NODE_H
#define MASK_IMAGE_FROM_SCAN_NODE_H

/// PROJECT
#include <csapex/model/node.h>
#include <opencv2/core/core.hpp>
#include <utils_laser_processing/data/scan.h>
#include <csapex_transform/transform_message.h>


namespace csapex {

class MaskImageFromScanNode : public csapex::Node
{
public:
    MaskImageFromScanNode();

public:
    virtual void process();
   virtual void setup(csapex::NodeModifier& node_modifier) override;
   virtual void setupParameters(Parameterizable& parameters) override;

protected:

    /// APEX
    Input            *input_scan_;
    Input            *input_image_;
    Input            *input_transform_;
    Output           *output_mask_;
    Output           *output_debug_;

    /// ALGORITHM
    /**
     * @brief creates a mask from a wall with given height by moving the laser points to the top and bottom of the wall and projecting them in an image
     * @param scan the complete laserscan
     * @param tf the transfromation from the camera to the laser (the laser seen from the camera c_T_L)
     * @param img_in the image which shall be masked
     * @param img_debug debug output that shows the projected laser points and proejcted mask points that fit on the image
     * @param mask outpu the mask (cv::Scalar::all(255) for points on the wall)
     * @param par_wall_height - input parameter the wall height from the bottom
     * @param par_laser_height - input parameter height of the laser points above the bottom
     * @param par_laser_opening_angle - the opening angle of the laserpoints the should be used to create the mask. (depends on the opening angle of the camera lense)
     */
    void createMaskFromLaser(const lib_laser_processing::Scan &scan,
                             const tf::Transform &tf,
                             const cv::Mat &img_in,
                             cv::Mat &img_debug,
                             cv::Mat &mask,
                             double par_wall_height = 0.4,
                             double par_laser_height = 0.13,
                             double par_laser_opening_angle = 2*0.34
                     );

    void loadCameraParameterFromFile(std::string filename,
                                    cv::Mat &camera_matrix,
                                    cv::Mat &distortion_coeffs);

    cv::Mat camera_matrix_; // intrinsic parameter matrix
    cv::Mat distortion_coeffs_;

    void loadCameraParameterFromFile();
};

} // csapex namespace


#endif // MASK_IMAGE_FROM_SCAN_NODE_H

