/// HEADER
#include "register_plugin.h"

/// COMPONENT
#include <sick14/target_message.h>

/// PROJECT
#include <csapex/msg/message_factory.h>
#include <csapex/model/tag.h>
#include <csapex_ros/ros_message_conversion.h>
#include <csapex_ros/ros_handler.h>
#include <csapex/msg/output.h>

/// SYSTEM
#include <boost/bind.hpp>
#include <csapex/utility/register_apex_plugin.h>
#include <sick_msgs/Target.h>
#include <boost/signals2.hpp>
#include <tf/tf.h>

CSAPEX_REGISTER_CLASS(csapex::RegisterSickPlugin, csapex::CorePlugin)

using namespace csapex;

RegisterSickPlugin::RegisterSickPlugin()
{
}

struct TargetConverter
{
    static connection_types::TargetMessage::Ptr ros2apex(const sick_msgs::Target::ConstPtr &ros_msg) {
        connection_types::TargetMessage::Ptr out(new connection_types::TargetMessage(ros_msg->header.frame_id, ros_msg->header.stamp.toNSec()));
        tf::poseMsgToTF(ros_msg->pose, out->transformation.value);
        out->frame_id = ros_msg->header.frame_id;
        return out;
    }
    static sick_msgs::Target::Ptr apex2ros(const connection_types::TargetMessage::ConstPtr& apex_msg) {
        sick_msgs::Target::Ptr out(new sick_msgs::Target);
        tf::poseTFToMsg(apex_msg->transformation.value, out->pose);
        out->header.frame_id = apex_msg->frame_id;

        if(apex_msg->stamp_micro_seconds == 0) {
            out->header.stamp = ros::Time::now();
        } else {
            out->header.stamp = out->header.stamp.fromNSec(apex_msg->stamp_micro_seconds);
        }
        return out;
    }
};

void RegisterSickPlugin::init(CsApexCore& core)
{
    RosMessageConversion::registerConversion<sick_msgs::Target, connection_types::TargetMessage, TargetConverter>();
}
