#ifndef CROSS_DETECTION_H
#define CROSS_DETECTION_H

#include <csapex/model/node.h>
#include <csapex_core_plugins/vector_message.h>
#include <csapex_vision/cv_mat_message.h>
#include <csapex_vision/roi_message.h>

namespace csapex {

class CrossDetection : public csapex::Node
{
public:
    CrossDetection();
    virtual void process();
   virtual void setup(csapex::NodeModifier& node_modifier) override;

private:
    struct Result
    {
        cv::Point2i point;
        cv::Point2i center;
        bool cross;
        bool found;
        int roi_size;
        bool roi_intersects;
    };

    double rho_;
    double theta_;
    int threshold_;
    double line_ratio_;
    double gap_ratio_;

//    int low_;
//    int high_;

    int tol_;

    Output* out_cross_;
    Output* out_roi_;
    Output* out_debug_;

    Input* in_mat_;
    Input* in_rois_;
    Input* in_edges_;

    void detectTarget(const cv::Mat &img, u_int64_t stamp, const cv::Mat &edges, std::vector<Result> &results);

    Result detectCross(cv::Mat &out, const std::vector<cv::Vec4i> &lines, const Roi &roi);
    void detectLine(cv::Mat &out, const std::vector<cv::Vec4i> &lines, const Roi &roi, Result &res);

    void filterLines(cv::Mat &out, std::vector<cv::Vec4i> &lines, int r_x, int r_y, int r_w, int r_h, int w, int h);

    void update();
};
}

#endif // CROSS_DETECTION_H
