#ifndef TARGET_FINDER_H
#define TARGET_FINDER_H

#include <csapex/model/node.h>
#include <ctime>
#include <tf/tf.h>
#include <opencv2/core/core.hpp>

#include <csapex_scan_2d/scan_message.h>
#include <csapex_transform/transform_message.h>
#include <utils_laser_processing/data/scan.h>
#include <sick14/cross_message.h>

#include <boost/circular_buffer.hpp>

namespace csapex {

class TargetFinder : public csapex::Node
{
public:
    TargetFinder();

   virtual void setup(csapex::NodeModifier& node_modifier) override;
    virtual void process();

private:
    Input *in_laser_;
    Input *in_target_pos_;
    Input *input_transform_cam_base_link_;
    Input *input_transform_scan_cam_;
    Output *out_target_pose_;
    Output *out_line_active_;

    bool has_camera_params_;
    cv::Mat camera_matrix_;
    cv::Mat distortion_coeffs_;
    
    connection_types::ScanMessage::ConstPtr scan_;

    //! Transform and publish point
    void processTarget(connection_types::CrossMessage::ConstPtr target);

    void updateConfig();
};

}

#endif // TARGET_FINDER_H
